// DJTDemo.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <conio.h>
#include "DJTDemo.h"
#include "djtsatlib.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define PI  (3.1415926535897932384626433832795)

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

using namespace std;

double UTCDateTime()
{
	double t;
	t = static_cast<double>(time(NULL));
	t /= 60 * 60 * 24;
	t += 70 * 365 + 19; //70 year's
	return t;
}

void KeyWait()
{
	puts("Press any key to continue!");
	_getch();
}

double ToDeg(double rads)
{
	return (rads * 180.0) / PI;
}

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		return 1;
	}

	puts("Simple console demo to show the use of David J. Taylors SGP4 COM component\n"
		"Program by Peter De Groot ON1CIM 10-Jan-2001.\n"
		"(To create the 'ISatellites' helper class:\n"
		"Classwizard--> Create new Class-->from typelib-->point ot *.tlb or *.dll file\n"
		"DJTSatLib.dll or *.tlb. The helper class will be created)\n");

	CoInitialize(NULL);	// initialize the COM stuff

	// Get the CLSID for the IDispatch interface
	CLSID clsid;
	::CLSIDFromProgID(L"DJTSatLib.Satellites", &clsid);
	//create the component and attach the IDispatch interface
	ISatellites	m_oSat;
	m_oSat.CreateDispatch(clsid);

	//now we are ready to rock...
	//set the path for the kepler files (the current directory in this case)
	m_oSat.SetKeplerPath(_T("."));
	//print version number from the component
	printf("DJTSatlib component version: %s\n", m_oSat.GetVersionString());

	//set observer latitude,longitude, height
	m_oSat.SetObsHeight(36.0);
	m_oSat.SetObsLat(51.08578);
	m_oSat.SetObsLon(4.8870475);

	//and read it back for display...
	printf("Observer location, Lon: %3.3f, lat: %3.3f, Height: %.1fm\n",
			m_oSat.GetObsLon(),
			m_oSat.GetObsLat(),
			m_oSat.GetObsHeight());

	//how many satellites did the kepler files contain and display them
	printf("Number of satellites: %d\n", m_oSat.GetSatCount());
	printf("Satellites list: %s\n\n", m_oSat.GetSatNames());
	KeyWait();

	// Create a passlist for 'NOAA 14' with a duration of minimum 180s for today visible 5 deg aove horizon.
	CString PassList;
	PassList = m_oSat.PassListForPeriod("NOAA 14", UTCDateTime(), 180, 1, 5.0);
	//change the ',' with 'newline' to display the list...
	PassList.Replace(",", "\n");
	printf("PassList for 'NOAA 12' , min 180s, 5.0 above horizon:\n%s\n\n",
			PassList);
	KeyWait();

	BOOL IsVisible,southb;
	double lat,lon,azim,elev,phase,range,rangerate,alti;
	//Get visible and position of the SUN
	IsVisible = m_oSat.IsVisible("SUN", m_oSat.DateTimeToJT(UTCDateTime()),
		0.0, &southb, &azim, &elev, &lon, &lat, &range, &rangerate, &alti, &phase);
	printf("SUN is %s, azim: %3.2f, elev: %3.2f\n\n",
		(IsVisible)?"Visible":"Not visible",
		ToDeg(azim),
		ToDeg(elev));
	//thesame for the MOON now, do also display the moons phase.
	IsVisible = m_oSat.IsVisible("MOON", m_oSat.DateTimeToJT(UTCDateTime()),
		0.0, &southb, &azim, &elev, &lon, &lat, &range, &rangerate, &alti, &phase);
	printf("MOON is %s, azim: %3.2f, elev: %3.2f, Phase: %.1f.\n\n",
		(IsVisible)?"Visible":"Not visible",
		ToDeg(azim),
		ToDeg(elev),
		phase);
	//now do it for 'NOAA 14' display range, rangrate and altitude.
	IsVisible = m_oSat.IsVisible("NOAA 14", m_oSat.DateTimeToJT(UTCDateTime()),
		5.0, &southb, &azim, &elev, &lon, &lat, &range, &rangerate, &alti, &phase);
	printf("NOAA 14 is %s, azim: %3.2f, elev: %3.2f\nRange: %.1fkm, Rangerate: %.1fkm/s, Altitude: %.1fkm\n",
		(IsVisible)?"Visible":"Not visible",
		ToDeg(azim),
		ToDeg(elev),
		range,
		rangerate,
		alti);

	//disconnect component
	m_oSat.DetachDispatch();
	//release COM stuff
	CoUninitialize();

	//thats all folks....
	KeyWait();
	return 0;
}


