' Windows Script Host Sample Script for David's Active-X DLL Satellite Library
'
' 2001 Jan 09	Version 3.0.0
'
' This script requires the Windows Scripting Host to run.  This comes with 
' Windows 98 and Windows 2000, and Microsoft Internet Explorer 4 and above.
' If you do not have the latest version of WSH you can download it from the
' Microsoft Web site.
'
' The Active-X DLL "DJTSatLib.dll" must be installed and registered with the
' operating system.  To do this, from the command prompt, change directory to
' the folder containing DJTSatLib.dll and enter the command:
'
'	REGSVR32 DJTSatLib.dll
'
' You should receive the message "DllRegisterServer in DJTSatLib.dll suceeded"
'
'
' This sample code sets up a connection to the server library, connects the 
' library to your Kepler elements, retrieves all the NOAA satellites, defines 
' an observer location, and obtains the passes for NOAA 15 on a particular day.
' This is intended purely as a demonstration of how to use the library.
' Similar steps could be carried out in any language, or even on an 
' Active Server Pages (ASP) Web page, enabling predictions over the Internet!
'
' The pass-list entries are single-space separated elements:
' start time  direction  longitude  elevation  seconds  sat-name
' 0203081858      S         359        79        874     NOAA 15
'
'
' The only function not shown here is the encapsulated SPG4 routine, which will
' determine if a satellite is visible or not.  This function is defined as:
'
'	function name:	IsVisible 
'
'	parameters:	Name: WideString
'			JulianTime: double
'			ThresholdDeg: Double
'			out Southbound: WordBool
'			out Azimuth: Double
'			out Elevation: Double
'			out Latitude: Double
'			out Longitude: Double
'
'	function type:	WordBool
'
' Please see the source file DJTSatellites.pas for a complete definition of
' all functions and properties defined by this library.


' ------------------------ Start of Sample VBS code -------------------------------
'
' Create a connection to an instance of the Satellites object
Set objSatList = WScript.CreateObject ("DJTSatLib.Satellites")

' Get the library version
lib_version = objSatList.VersionString

' Load the Kepler 2-line element data files by setting
' the KeplerPath variable
objSatList.KeplerPath = "C:\Davids\Delphi5\Radio\WXtrack\Source"

' Define the location of the observer
objSatList.ObsLat = 55.9		' degrees
objSatList.ObsLon = -3.2		' degrees
objSatList.ObsHeight = 180		' metres

' Define the day we want to start at, note all times are UTC
start = DateSerial (2001, 1, 9)		' 2001 January 09
days = 1				' number of days to predict
min_elevation = 2			' degrees above the horizon
min_seconds = 300			' required min pass duration

' Get a comma-separated list of all available satellites
sat_names = objSatList.SatNames
' Remove any remaining '"' marks
sat_names = Replace (sat_names, Chr(34), "")
' Split the comma-separated list into an array
name_array = Split (sat_names, ",", -1)

' Display those satellites starting "NOAA"
noaa_list = ""
For Each name In name_array
  If Left (name, 4) = "NOAA" Then noaa_list = noaa_list & name & "  "
Next

status = MsgBox (noaa_list, _
		vbInformation, _
		"DJT Satellite Library V" & lib_version & " - NOAA satellites found")

' Read the data back from the Satellites object
lat = objSatList.ObsLat
lon = objSatList.ObsLon
' Convert to more natural longitude format
If lon > 180 Then lon = lon - 360

' Generate a pass-list for the period
passes = objSatList.PassListForPeriod _
		("NOAA 15", start, min_seconds, days, min_elevation)

' Convert the pass-list into multiple lines, using CR-LF as a delimiter.
passes = Replace (passes, ",", vbCRLF)
' Strip out the quotation marks for a better display
passes = Replace (passes, Chr(34), "")

status = MsgBox ("Observer at latitude: " & FormatNumber (lat, 2) & _
		"  longitude: " & FormatNumber (lon, 2) & "" _
		& vbCRLF & vbCRLF & passes,    _   
		vbInformation,       _
		"Passes for NOAA 15 for " & FormatDateTime (start, vbLongDate))

' Free the object now it is finished with
Set objSatList = Nothing

' ------------------------ End of Sample VBS code ---------------------------------
