unit TestForm;

{
2001 Jan 07  First version
             Correct for extra parameter Phase returned from IsVisible
2001 Jan 08  Update for Range, RangeRate and Alitude being returned
2001 May 15  Display new orbit number and phase results
             Don't produce errors if the library hasn't been registered
             State source folder locations used in the ReadMe.txt file
2001 May 17  Update mean anomaly to simply return real number of orbits
2001 Sep 08  Update version number info to 3.1.2.19
2002 Aug 17  Add display of period
2002 Aug 19  Add some more goodies for display
2002 Aug 23  Check for V3.3 of library as a minimum
             Add choice of Kepler data time (as well as real-time and manual)
             Add Sun/Moon display
2002 Aug 24  Add Kepler Mean Anomaly
2004 Jul 21  Update for PassDetails display
2004 Aug 03  Trap error if earlier DLL is in use
             Re-arrange routines into alphabetical order
2004 Dec 19  Define inital satellite text in form's OnShow event
             Minor cosmetic update
2005 Apr 14  Add IgnoreRefraction test
2005 Jul 17  Use to check MakePassDetails for bad input
2006 Sep 02  Update for MakePassDetails2 method in V3.6
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DJTSatLib_TLB, StdCtrls, ComCtrls, ExtCtrls, VersInfo;

type
  TFormMain = class (TForm)
    LabelVersion: TLabel;
    LabelKeplerPath: TLabel;
    ListBoxSatNames: TListBox;
    LabelNames: TLabel;
    LabelLocation: TLabel;
    ListBoxPasses: TListBox;
    LabelPasses: TLabel;
    ButtonMakePasses: TButton;
    LabelChoose: TLabel;
    DateTimePickerDate: TDateTimePicker;
    EditSatellite: TEdit;
    DateTimePickerTime: TDateTimePicker;
    LabelUTC: TLabel;
    LabelSatState: TLabel;
    ButtonClose: TButton;
    LabelVisible: TLabel;
    LabelPassDetails: TLabel;
    TimerRealTime: TTimer;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    DateTimePickerPasses: TDateTimePicker;
    LabelPassDate: TLabel;
    LabelSatName: TLabel;
    LabelAOSLOS: TLabel;
    LabelPeriod: TLabel;
    LabelJT: TLabel;
    ComboBoxTimeframe: TComboBox;
    LabelSunMoon: TLabel;
    LabelDetails: TLabel;
    ListBoxPassDetails: TListBox;
    BevelAOSLOS: TBevel;
    CheckBoxIgnoreRefraction: TCheckBox;
    CheckBoxShowRangeRate: TCheckBox;
    procedure ButtonCloseClick (Sender: TObject);
    procedure ButtonMakePassesClick (Sender: TObject);
    procedure ComboBoxTimeframeChange (Sender: TObject);
    procedure DateTimePickerDateChange (Sender: TObject);
    procedure FormCreate (Sender: TObject);
    procedure FormDestroy (Sender: TObject);
    procedure FormShow (Sender: TObject);
    procedure ListBoxSatNamesClick (Sender: TObject);
    procedure ListBoxPassesClick (Sender: TObject);
    procedure TimerRealTimeTimer (Sender: TObject);
    procedure CheckBoxIgnoreRefractionClick(Sender: TObject);
  private
    { Private declarations }
    ISat: ISatellites;
    procedure DateTimeChanged (const truncate_msec: boolean);
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

implementation

uses
  ComObj;

{$R *.DFM}

const
  latitude = 56.0;
  longitude = -3.2;
  obs_height = 180.0;
  expected = 'Lon: -3.2  Lat: 56  Height: 180m';


function NowUTC: TDateTime;
var
  system_datetime: TSystemTime;
begin
  GetSystemTime (system_datetime);
  Result := SystemTimeToDateTime (system_datetime);
end;


procedure TFormMain.ButtonCloseClick (Sender: TObject);
begin
  Close;
end;


procedure TFormMain.ButtonMakePassesClick (Sender: TObject);
begin
  if Assigned (ISat) then
    ListBoxPasses.Items.CommaText := ISat.PassListForPeriod (
      EditSatellite.Text,
      DateTimePickerPasses.DateTime,
      120, 2, 1.5);
end;


procedure TFormMain.CheckBoxIgnoreRefractionClick (Sender: TObject);
var
  hint: string;
begin
  if Assigned (ISat)
    then ISat.IgnoreRefraction := CheckBoxIgnoreRefraction.Checked;
  ComboBoxTimeframeChange (Self);
  ButtonMakePasses.Click;
  if CheckBoxIgnoreRefraction.Checked
    then hint := 'Atmospheric refraction is ignored when calculating elevation'
    else hint := 'Atmospheric refraction is included when calculating elevation';
  CheckBoxIgnoreRefraction.Hint := hint;
end;


procedure TFormMain.ComboBoxTimeframeChange (Sender: TObject);
var
  now_dt, keps_dt: TDateTime;
  offset: double;
  keps_epoch: double;
  sat: string;
  dummy: double;
begin
  TimerRealTime.Enabled := False;
  case ComboBoxTimeframe.ItemIndex of
    0: begin
       DateTimePickerDate.DateTime := NowUTC;
       DateTimePickerTime.DateTime := NowUTC;
       TimerRealTime.Enabled := True;
       end;
    1: begin
       end;
    2: if Assigned (ISat) then
         begin
         now_dt := NowUTC;
         offset := ISat.DateTimeToJT (now_dt) - now_dt;
         sat := EditSatellite.Text;
         if ISat.VersionString >= '3.3' then
           begin
           ISat.KeplerData (sat, keps_epoch, dummy, dummy, dummy, dummy,
                            dummy, dummy, dummy);
           keps_dt := keps_epoch - offset;
           DateTimePickerDate.DateTime := keps_dt;
           DateTimePickerTime.DateTime := keps_dt;
           DateTimeChanged (False);
           end;
         end;
  end;
end;


procedure TFormMain.DateTimeChanged (const truncate_msec: boolean);

  // Helmut Strickner test code routines
  {
    example for current MeanAnomaly in degrees:
    ma : mean anomaly from keps data
    mm : mean motion from keps data
    t : time since epoch time from keps data
    decay : decay from keps data
  }
  // Returns a result in range 0..360 degrees
  function degr (x: double): double;
  var
    index: integer;
  begin
    if x < 0 then index := 360 else index:= 0;
    Result := (x - (Int (x / 360.0) * 360.0)) + index;
  end;

  // Calculate drag coefficient
  function drag (decay, mm, t: double): double;
  begin
    Result := decay * t / (mm * 3);
  end;

  // Returns actual Mean Motion with decay
  function mm_new_ (decay, mm, t: double): double;
  begin
    Result := mm - 2 * decay * t * t;
  end;

  // Returns actual Mean Anomaly with or without decay
  // F = 1 without or f = (1 - 3 * dt) with decay
  function ma_new_ (ma, mm, t, f: double): double;
  begin
    Result := degr (ma + 360 * mm * t * f);
  end;
  // End of Helmut Strickner test code

const
  Rad2Deg = 180 / pi;
var
  utc: TDateTime;
  hour, min, sec, msec: word;
  jt: double;
  sat: string;
  is_visible: WordBool;
  southbound: WordBool;
  az, el, lon, lat, phase: double;
  st: string;
  range, range_rate, altitude: double;
  rev_number: integer;
  orbit_phase: double;
  ma: double;
  ma_number: integer;
  ma_phase: double;
  aos, los: TDateTime;
  x, y, z, xdot, ydot, zdot: double;
  per_hours, per_mins: double;
  ma_k, j_epoch, incl, raan, ecc, omegao, decay_rate, period: double;
  solar_data: string;
  // Helmut Strickner stuff
  mm: double;
  epoch_elapsed: double;
  MM_new: double;
  dt: double;
  MA_new: double;
begin
  if not Assigned (ISat) then Exit;
  utc := Trunc (DateTimePickerDate.DateTime) +
         Frac (DateTimePickerTime.DateTime);
  if truncate_msec then
    begin
    DecodeTime (utc, hour, min, sec, msec);
    utc := Trunc (DateTimePickerDate.DateTime) +
           EncodeTime (hour, min, sec, 0);
    end;
  jt := ISat.DateTimeToJT (utc);
  LabelJT.Caption := FormatDateTime ('"UTC: "hh:nn:ss.zzz',  utc) +
                     Format (',  Julian date: %.8f', [jt]);
  sat := EditSatellite.Text;
  mm := 0;
  MA_new := 0;
  LabelPeriod.Caption := 'This information requires library V3.3 or greater';
  if ISat.VersionString >= '3.3' then
    begin
    //  per_hours := 24.0 * ISat.Period (sat);  // just for period
    // This call does the works!
    ISat.KeplerData (sat, j_epoch, incl, raan, ecc, omegao, decay_rate, period, ma_k);
    // Convert to revolutions per day per day
    decay_rate := decay_rate * Sqr (1440) / (2 * pi);
    // Compute mean motion
    mm := 1 / period;
    period := 24 * period;         // Convert to hours
    per_mins := 60.0 * Frac (period);
    per_hours := Trunc (period);
    ma_k := Rad2Deg * ma_k;        // Convert to degrees
    LabelPeriod.Caption := Format ('Per: %.0fh %.0fm', [per_hours, per_mins]) +
      ', Jul epoch: ' + FloatToStr (j_epoch) +
      ', Incl (deg): ' + Format ('%.4f', [Rad2Deg * incl]) +
      ', RAAN (deg): ' + Format ('%.4f', [Rad2Deg * raan]) +
      ', Ecc: ' + FloatToStr (ecc) +
      ', ArgOfPer (deg): ' + Format ('%.4f', [Rad2Deg * omegao]) +
      ', DecayRate: ' + FloatToStr (decay_rate) +
      ', MA: ' + Format ('%f', [ma_k]);
    end;
  is_visible := ISat.IsVisible (sat, jt, 0.5, southbound, az, el, lon, lat,
                                range, range_rate, altitude, phase);
  // Must collect orbit information immediately after the IsVisible call
  rev_number := ISat.RevNumber;
  orbit_phase := ISat.OrbitPhase;
  ISat.SatState (x, y, z, xdot, ydot, zdot);
  ma := ISat.MeanAnomaly;    // Get the mean anomaly, specified in revolutions
  ma_number := Trunc (ma);         // ..get the orbit number
  ma_phase := 2 * pi * Frac (ma);  // ..and the mean anomaly in this orbit
  // Helmut Strickner stuff - need to check if we got a valid mm
  if mm <> 0 then
    begin
    // Calculate epoch difference between KeplerData and present time in decimal days
    epoch_elapsed := (jt - j_epoch);
    // Calculate actual mm with decay rate
    MM_new := mm_new_ (decay_rate, mm, epoch_elapsed);
    // Calculate decay constant from elapsed time
    dt := drag (decay_rate, MM_new, epoch_elapsed);
    // Calculate actual ma with decay rate, using the MA from the Keplers
    MA_new := ma_new_ (ma_k, MM_new, epoch_elapsed, (1 - 3 * dt));
    end;
  LabelPassDetails.Caption := Format
    ('Az: %2f  El: %.2f,  Lon: %.2f  Lat: %.2f,  ' +
     'Range: %0.fkm  Range rate: %.1fkm/s  Altitude: %.2fkm,  ' +
     'Revolution: %d,  Orbit phase: %.6f,  Mean anomaly: %d + %.2f,  ' +
     'MA: %d/256,  MA(new): %.2f',
    [Rad2Deg*az, Rad2Deg*el, Rad2Deg*lon, Rad2Deg*lat,
     range, range_rate, altitude,
     rev_number, Rad2Deg*orbit_phase, ma_number, Rad2Deg*ma_phase,
     Round (ma_phase * 256 / (2 * pi)), MA_new]);
  LabelSatState.Caption := Format
              ('x:%5.0f km  y:%5.0f km  z:%5.1f km,  ' +
               'x:%2.3f km/s  y:%2.3f km/s  z:%2.3f km/s',
               [x, y, z, xdot, ydot, zdot]);
  if is_visible
    then st := sat + ' is visible, '
    else st := sat + ' is not visible, ';
  LabelSatName.Caption := sat + ' AOS/LOS';
  aos := ISat.FindAOS (sat, utc);
  los := ISat.FindLOS (sat, aos);
  LabelAOSLOS.Caption := DateTimeToStr (aos) + #13 + DateTimeToStr (los);
  solar_data := '';
  if ISat.IsVisible ('SUN', jt, 0.5, southbound, az, el, lon, lat, range, range_rate, altitude, phase)
    then st := st + 'SUN is visible, '
    else st := st + 'SUN is not visible, ';
  solar_data := 'Sun' + #13 + Format ('  Az: %2f  El: %.2f', [Rad2Deg*az, Rad2Deg*el]) +
                        #13 + Format ('  Lon: %.2f  Lat: %.2f', [Rad2Deg*lon, Rad2Deg*lat]) + #13 + #13;
  if ISat.IsVisible ('MOON', jt, 0.5, southbound, az, el, lon, lat, range, range_rate, altitude, phase)
    then LabelVisible.Caption := st + 'MOON is visible'
    else LabelVisible.Caption := st + 'MOON is not visible';
  solar_data := solar_data + 'Moon' + #13 + Format ('  Az: %2f  El: %.2f', [Rad2Deg*az, Rad2Deg*el]) +
                        #13 + Format ('  Lon: %.2f  Lat: %.2f', [Rad2Deg*lon, Rad2Deg*lat]);
  LabelSunMoon.Caption := solar_data;
end;


procedure TFormMain.DateTimePickerDateChange (Sender: TObject);
begin
  DateTimeChanged (True);
end;


procedure TFormMain.FormCreate (Sender: TObject);
var
  lon: double;
begin
  try
    // Try and connect to the Active-X DLL
    ISat := CreateComObject (CLASS_Satellites) as Satellites;
  except
  end;

  if Assigned (ISat)
  then
    begin
    // Get the DLL version information string
    LabelVersion.Caption := 'DLL version: ' + ISat.VersionString;
    CheckBoxShowRangeRate.Enabled := ISat.VersionString >= '3.6';
    // Tell the DLL to read the Kepler files in the folder named
    ISat.KeplerPath := 'C:\Davids\Delphi5\Radio\WXtrack\Source\';

    // This could equally be a file like this
//    ISat.KeplerPath := 'C:\Davids\Delphi5\Radio\WXtrack\Source\_misc.2li';
    // All files in that folder with the extensions .2LI, .TLE and .TXT
    // would be scanned for valid Kepler data

    // Find out how many satellites were found
    LabelKeplerPath.Caption := 'Kepler path: ' + ISat.KeplerPath +
        '  which contains ' + IntToStr (ISat.SatCount) + ' satellites';
    // Get comma-separated list of the satellite names
    ListBoxSatNames.Items.CommaText := ISat.SatNames;
    // Try setting and reading back the observer location
    ISat.ObsLon := longitude;
    ISat.ObsLat := latitude;
    ISat.ObsHeight := obs_height;
    lon := ISat.ObsLon;
    if lon > 180.0 then lon := lon -360.0;
    LabelLocation.Caption := Format
       ('Location:  Lon = %f  Lat = %f  Height = %fm  (expected %s)',
        [lon, ISat.ObsLat, ISat.ObsHeight, expected]);
    CheckBoxIgnoreRefraction.Enabled := ISat.VersionString >= '3.5'
    end
  else
    begin
    ShowMessage ('Error connecting to in-process COM server DJTSatLib.dll' + #13 + #13 +
                 'Try using the command "REGSVR32 DJTSatLib.dll" ' +
                 'from the command prompt in the folder containing ' +
                 'the file DJTSatLib.dll.' + #13 + #13 +
                 'The REGSVR32 command may be found in your Windows System folder.');
    end;
end;


procedure TFormMain.FormDestroy (Sender: TObject);
begin
  if Assigned (ISat) then
    begin
    ISat.KeplerPath := '';
    ISat := nil;
    end;
end;


procedure TFormMain.FormShow (Sender: TObject);
begin
  DesktopFont := True;
  EditSatellite.Text := 'ORBCOMM FM 34'; 
  ComboBoxTimeframe.ItemIndex := 0;
  DateTimePickerPasses.DateTime := NowUTC;
  ButtonMakePasses.Click;
  DateTimeChanged (True);
end;


procedure TFormMain.ListBoxPassesClick (Sender: TObject);
const
  delta = 30;
var
  index: integer;
  pass: string;
begin
  index := ListBoxPasses.ItemIndex;
  if index <> -1 then
    begin
    ListBoxPassDetails.Clear;
    pass := ListBoxPasses.Items.Strings [index];
    if ISat.VersionString >= '3.4'
    then
      begin
      if CheckBoxShowRangeRate.Checked
      then
        begin
        LabelDetails.Caption := 'Details: date/time  az  el  range  range-rate  nadir-angle';
        ListBoxPassDetails.Items.CommaText := ISat.MakePassDetails2 (pass, delta);
        end
      else
        begin
        LabelDetails.Caption := 'Pass details: date/time  az  el  range  nadir-angle';
        ListBoxPassDetails.Items.CommaText := ISat.MakePassDetails (pass, delta);
        end
      end
    else
      ShowMessage ('Version 3.4 or higher of the DLL is required');
    end;
end;


procedure TFormMain.ListBoxSatNamesClick (Sender: TObject);
var
  index: integer;
begin
  index := ListBoxSatNames.ItemIndex;
  if index = -1 then Exit;
  EditSatellite.Text := ListBoxSatNames.Items.Strings [index];
  ComboBoxTimeframeChange (Self);
end;


procedure TFormMain.TimerRealTimeTimer (Sender: TObject);
begin
  TimerRealTime.Enabled := False;
  DateTimePickerDate.DateTime := NowUTC;
  DateTimePickerTime.DateTime := NowUTC;
  DateTimeChanged (True);
  TimerRealTime.Enabled := ComboBoxTimeframe.ItemIndex = 0;
end;


end.

