unit TestForm;

{
2001 Jan 07  First version
             Correct for extra parameter Phase returned from IsVisible
2001 Jan 08  Update for Range, RangeRate and Alitude being returned
2001 May 15  Display new orbit number and phase results
             Don't produce errors if the library hasn't been registered
             State source folder locations used in the ReadMe.txt file
2001 May 17  Update mean anomaly to simply return real number of orbits
2001 Sep 08  Update version number info to 3.1.2.19
2002 Feb 06  Update for Delphi 6 (D6 personal checked)
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DJTSatLib_TLB, StdCtrls, ComCtrls, ExtCtrls;

type
  TFormMain = class (TForm)
    LabelVersion: TLabel;
    LabelKeplerPath: TLabel;
    ListBoxSatNames: TListBox;
    LabelNames: TLabel;
    LabelLocation: TLabel;
    ListBoxPasses: TListBox;
    LabelPasses: TLabel;
    ButtonMakePasses: TButton;
    LabelChoose: TLabel;
    DateTimePickerDate: TDateTimePicker;
    EditSatellite: TEdit;
    DateTimePickerTime: TDateTimePicker;
    LabelUTC: TLabel;
    LabelSatState: TLabel;
    ButtonClose: TButton;
    LabelVisible: TLabel;
    LabelPassDetails: TLabel;
    CheckBoxRealTime: TCheckBox;
    TimerRealTime: TTimer;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    DateTimePickerPasses: TDateTimePicker;
    LabelPassDate: TLabel;
    LabelSatName: TLabel;
    LabelAOSLOS: TLabel;
    procedure ButtonCloseClick (Sender: TObject);
    procedure ButtonMakePassesClick (Sender: TObject);
    procedure CheckBoxRealTimeClick (Sender: TObject);
    procedure DateTimePickerDateChange (Sender: TObject);
    procedure FormCreate (Sender: TObject);
    procedure FormDestroy (Sender: TObject);
    procedure FormShow (Sender: TObject);
    procedure ListBoxSatNamesClick (Sender: TObject);
    procedure TimerRealTimeTimer (Sender: TObject);
  private
    { Private declarations }
    ISat: ISatellites;
    procedure date_time_changed;
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

implementation

uses
  ComObj;

{$R *.DFM}

const
  latitude = 56.0;
  longitude = -3.2;
  obs_height = 180.0;
  expected = 'Lon: -3.2  Lat: 56  Height: 180m';


function NowUTC: TDateTime;
var
  system_datetime: TSystemTime;
begin
  GetSystemTime (system_datetime);
  Result := SystemTimeToDateTime (system_datetime);
end;


procedure TFormMain.ButtonCloseClick (Sender: TObject);
begin
  Close;
end;


procedure TFormMain.ButtonMakePassesClick (Sender: TObject);
begin
  if Assigned (ISat) then
    ListBoxPasses.Items.CommaText := ISat.PassListForPeriod (
      EditSatellite.Text,
      DateTimePickerPasses.DateTime,
      120, 2, 1.5);
end;


procedure TFormMain.CheckBoxRealTimeClick (Sender: TObject);
begin
  TimerRealTime.Enabled := CheckBoxRealTime.Checked;
end;


procedure TFormMain.DateTimePickerDateChange (Sender: TObject);
begin
  date_time_changed;
end;


procedure TFormMain.date_time_changed;
const
  Rad2Deg = 180 / pi;
var
  utc: TDateTime;
  jt: double;
  sat: string;
  is_visible: WordBool;
  southbound: WordBool;
  az, el, lon, lat, phase: double;
  st: string;
  range, range_rate, altitude: double;
  rev_number: integer;
  orbit_phase: double;
  ma: double;
  ma_number: integer;
  ma_phase: double;
  aos, los: TDateTime;
  x, y, z, xdot, ydot, zdot: double;
begin
  if not Assigned (ISat) then Exit;
  utc := Trunc (DateTimePickerDate.DateTime) +
         Frac (DateTimePickerTime.DateTime);
  jt := ISat.DateTimeToJT (utc);
  sat := EditSatellite.Text;
  is_visible := ISat.IsVisible (sat, jt, 0.5, southbound, az, el, lon, lat,
                                range, range_rate, altitude, phase);
  // Must collect orbit information immediately after the IsVisible call
  rev_number := ISat.RevNumber;
  orbit_phase := ISat.OrbitPhase;
  ISat.SatState (x, y, z, xdot, ydot, zdot);
  ma := ISat.MeanAnomaly;    // Get the mean anomaly, specified in revolutions
  ma_number := Trunc (ma);         // ..get the orbit number
  ma_phase := 2 * pi * Frac (ma);  // ..and the mean anomaly in this orbit
  LabelPassDetails.Caption := Format
    ('Az:%1f  El:%.1f   Lon:%.1f  Lat:%.1f   ' +
     'Range:%0.fkm  Range rate:%.1fkm/s  Altitude:%.0fkm   ' +
     'Revolution:%d  Phase:%.1f  Mean anomaly: %d %.1f',
    [Rad2Deg*az, Rad2Deg*el, Rad2Deg*lon, Rad2Deg*lat, range, range_rate,
     altitude, rev_number, Rad2Deg*orbit_phase, ma_number, Rad2Deg*ma_phase]);
  LabelSatState.Caption := Format
              ('x:%5.0f km  y:%5.0f km  z:%5.0f km,   ' +
               'x:%2.3f km/s  y:%2.3f km/s  z:%2.3f km/s',
               [x, y, z, xdot, ydot, zdot]);
  if is_visible
    then st := sat + ' is visible, '
    else st := sat + ' is not visible, ';
  LabelSatName.Caption := sat + ' AOS/LOS';
  aos := ISat.FindAOS (sat, utc);
  los := ISat.FindLOS (sat, aos);
  LabelAOSLOS.Caption := DateTimeToStr (aos) + #13 + DateTimeToStr (los);
  if ISat.IsVisible ('SUN', jt, 0.5, southbound, az, el, lon, lat, range, range_rate, altitude, phase)
    then st := st + 'SUN is visible, '
    else st := st + 'SUN is not visible, ';
  if ISat.IsVisible ('MOON', jt, 0.5, southbound, az, el, lon, lat, range, range_rate, altitude, phase)
    then LabelVisible.Caption := st + 'MOON is visible'
    else LabelVisible.Caption := st + 'MOON is not visible';
end;


procedure TFormMain.FormCreate (Sender: TObject);
begin
  try
    // Try and connect to the Active-X DLL
    ISat := CreateComObject (CLASS_Satellites) as Satellites;
  except
  end;

  if Assigned (ISat)
  then
    begin
    // Get the DLL version information string
    LabelVersion.Caption := 'DLL version: ' + ISat.VersionString;
    // Tell the DLL to read the Kepler files in the folder named
    ISat.KeplerPath := 'C:\Davids\Delphi5\Radio\WXtrack\Source\';
    // and find out how many satellites were found
    LabelKeplerPath.Caption := 'Kepler path: ' + ISat.KeplerPath +
        '  which contains ' + IntToStr (ISat.SatCount) + ' satellites';
    // Get comma-separated list of the satellite names
    ListBoxSatNames.Items.CommaText := ISat.SatNames;
    // Try setting and reading back the observer location
    ISat.ObsLon := longitude;
    ISat.ObsLat := latitude;
    ISat.ObsHeight := obs_height;
    LabelLocation.Caption := Format
       ('Location:  Lon = %f  Lat = %f  Height = %fm  (expected %s)',
        [ISat.ObsLon, ISat.ObsLat, ISat.ObsHeight, expected]);
    end
  else
    begin
    ShowMessage ('Error connecting to in-process COM server DJTSatLib.dll' + #13 + #13 +
                 'Try using the command "REGSVR32 DJTSatLib.dll" ' +
                 'from the command prompt in the folder containing ' +
                 'the file DJTSatLib.dll.' + #13 + #13 +
                 'The REGSVR32 command may be found in your Windows System folder.');
    end;
end;


procedure TFormMain.FormDestroy (Sender: TObject);
begin
  if Assigned (ISat) then
    begin
    ISat.KeplerPath := '';
    ISat := nil;
    end;
end;


procedure TFormMain.FormShow (Sender: TObject);
begin
  DateTimePickerDate.DateTime := NowUTC;
  DateTimePickerTime.DateTime := NowUTC;
  DateTimePickerPasses.DateTime := NowUTC;
  ButtonMakePasses.Click;
  date_time_changed;
end;


procedure TFormMain.ListBoxSatNamesClick (Sender: TObject);
var
  index: integer;
begin
  index := ListBoxSatNames.ItemIndex;
  if index = -1 then Exit;
  EditSatellite.Text := ListBoxSatNames.Items.Strings [index];
end;


procedure TFormMain.TimerRealTimeTimer (Sender: TObject);
begin
  TimerRealTime.Enabled := False;
  DateTimePickerDate.DateTime := NowUTC;
  DateTimePickerTime.DateTime := NowUTC;
  date_time_changed;
  TimerRealTime.Enabled := CheckBoxRealTime.Checked;
end;

end.

