#include <jni.h>
#include "stdafx.h"
#include "jDJTSatLib.h"

COleDispatchDriver sat;

JNIEXPORT jboolean JNICALL Java_jDJTSatLib_init(JNIEnv *env, jobject obj)
{
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
		return false;

	CoInitialize(NULL);
	CLSID clsid;
	::CLSIDFromProgID(L"DJTSatLib.Satellites", &clsid);
	sat.CreateDispatch(clsid);

	return true;
}

JNIEXPORT void JNICALL Java_jDJTSatLib_deinit(JNIEnv *env, jobject obj)
{
	sat.DetachDispatch();
	CoUninitialize();
}

JNIEXPORT jstring JNICALL Java_jDJTSatLib_getVersionString(JNIEnv *env, jobject obj)
{
	CString result;
	sat.InvokeHelper(0x1, DISPATCH_PROPERTYGET, VT_BSTR, (void*)&result, NULL);
	return env->NewStringUTF(result.GetBuffer(0));
}

JNIEXPORT jstring JNICALL Java_jDJTSatLib_getKeplerPath(JNIEnv *env, jobject obj)
{
	CString result;
	sat.InvokeHelper(0x2, DISPATCH_PROPERTYGET, VT_BSTR, (void*)&result, NULL);
	return env->NewStringUTF(result.GetBuffer(0));
}

JNIEXPORT void JNICALL Java_jDJTSatLib_setKeplerPath(JNIEnv *env, jobject obj, jstring path)
{
	LPCTSTR Path = env->GetStringUTFChars(path, NULL);
	static BYTE parms[] = VTS_BSTR;
	sat.InvokeHelper(0x2, DISPATCH_PROPERTYPUT, VT_EMPTY, NULL, parms, Path);
	env->ReleaseStringUTFChars(path, Path);
}

JNIEXPORT jint JNICALL Java_jDJTSatLib_getSatCount(JNIEnv *env, jobject obj)
{
	jlong result;
	sat.InvokeHelper(0x3, DISPATCH_PROPERTYGET, VT_I4, (void*)&result, NULL);
	return result;
}

JNIEXPORT jstring JNICALL Java_jDJTSatLib_getSatNames(JNIEnv *env, jobject obj)
{
	CString result;
	sat.InvokeHelper(0x4, DISPATCH_PROPERTYGET, VT_BSTR, (void*)&result, NULL);
	return env->NewStringUTF(result.GetBuffer(0));
}

JNIEXPORT jdouble JNICALL Java_jDJTSatLib_getObsLat(JNIEnv *env, jobject obj)
{
	jdouble result;
	sat.InvokeHelper(0x5, DISPATCH_PROPERTYGET, VT_R8, (void*)&result, NULL);
	return result;
}

JNIEXPORT void JNICALL Java_jDJTSatLib_setObsLat(JNIEnv *env, jobject obj, jdouble newValue)
{
	static BYTE parms[] = VTS_R8;
	sat.InvokeHelper(0x5, DISPATCH_PROPERTYPUT, VT_EMPTY, NULL, parms, newValue);
}

JNIEXPORT jdouble JNICALL Java_jDJTSatLib_getObsLon(JNIEnv *env, jobject obj)
{
	jdouble result;
	sat.InvokeHelper(0x6, DISPATCH_PROPERTYGET, VT_R8, (void*)&result, NULL);
	return result;
}

JNIEXPORT void JNICALL Java_jDJTSatLib_setObsLon(JNIEnv *env, jobject obj, jdouble newValue)
{
	static BYTE parms[] = VTS_R8;
	sat.InvokeHelper(0x6, DISPATCH_PROPERTYPUT, VT_EMPTY, NULL, parms, newValue);
}

JNIEXPORT jdouble JNICALL Java_jDJTSatLib_getObsHeight(JNIEnv *env, jobject obj)
{
	jdouble result;
	sat.InvokeHelper(0x7, DISPATCH_PROPERTYGET, VT_R8, (void*)&result, NULL);
	return result;
}

JNIEXPORT void JNICALL Java_jDJTSatLib_setObsHeight(JNIEnv *env, jobject obj, jdouble newValue)
{
	static BYTE parms[] = VTS_R8;
	sat.InvokeHelper(0x7, DISPATCH_PROPERTYPUT, VT_EMPTY, NULL, parms, newValue);
}

JNIEXPORT jstring JNICALL Java_jDJTSatLib_PassListForPeriod(JNIEnv *env, jobject obj,
	jstring name, jdouble utcDate, jint minSeconds, jint numDays, jdouble minElevDegrees)
{
	LPCTSTR Name = env->GetStringUTFChars(name, NULL);
	CString result;
	static BYTE parms[] = VTS_BSTR VTS_R8 VTS_I4 VTS_I4 VTS_R8;

	sat.InvokeHelper(0x8, DISPATCH_METHOD, VT_BSTR, (void*)&result, parms, Name, utcDate,minSeconds,numDays,minElevDegrees);
	env->ReleaseStringUTFChars(name, Name);

	return env->NewStringUTF(result.GetBuffer(0));
}

JNIEXPORT jboolean JNICALL Java_jDJTSatLib_IsVisible(JNIEnv *env, jobject obj,
	jstring name, jdouble julianTime, jdouble thresholdDeg, jbooleanArray southBound,
	jdoubleArray azumuth, jdoubleArray elevation, jdoubleArray longitude, jdoubleArray latitude,
	jdoubleArray range, jdoubleArray rangeRate, jdoubleArray altitude, jdoubleArray phase)
{
	LPCTSTR Name = env->GetStringUTFChars(name, NULL);
	BOOL *Southbound = (BOOL*)env->GetBooleanArrayElements(southBound, 0);
	jdouble *Azumuth = env->GetDoubleArrayElements(azumuth, 0);
	jdouble *Elevation = env->GetDoubleArrayElements(elevation, 0);
	jdouble *Longitude = env->GetDoubleArrayElements(longitude, 0);
	jdouble *Latitude = env->GetDoubleArrayElements(latitude, 0);
	jdouble *Range = env->GetDoubleArrayElements(range, 0);
	jdouble *RangeRate = env->GetDoubleArrayElements(rangeRate, 0);
	jdouble *Altitude = env->GetDoubleArrayElements(altitude, 0);
	jdouble *Phase = env->GetDoubleArrayElements(phase, 0);

	BOOL result;
	static BYTE parms[] = VTS_BSTR VTS_R8 VTS_R8 VTS_PBOOL VTS_PR8 VTS_PR8 VTS_PR8 VTS_PR8 VTS_PR8 VTS_PR8 VTS_PR8 VTS_PR8;
	sat.InvokeHelper(0x9, DISPATCH_METHOD, VT_BOOL, (void*)&result, parms, Name, julianTime, thresholdDeg, Southbound, Azumuth, Elevation, Longitude, Latitude, Range, RangeRate, Altitude, Phase);

	env->ReleaseStringUTFChars(name, Name);
	env->ReleaseBooleanArrayElements(southBound, (jboolean*)Southbound, 0);
	env->ReleaseDoubleArrayElements(azumuth, Azumuth, 0);
	env->ReleaseDoubleArrayElements(elevation, Elevation, 0);
	env->ReleaseDoubleArrayElements(longitude, Longitude, 0);
	env->ReleaseDoubleArrayElements(latitude, Latitude, 0);
	env->ReleaseDoubleArrayElements(range, Range, 0);
	env->ReleaseDoubleArrayElements(rangeRate, RangeRate, 0);
	env->ReleaseDoubleArrayElements(altitude, Altitude, 0);
	env->ReleaseDoubleArrayElements(phase, Phase, 0);

	return result;
}

JNIEXPORT jdouble JNICALL Java_jDJTSatLib_DateTimeToJT(JNIEnv *env, jobject obj, jdouble dateTime)
{
	jdouble result;
	static BYTE parms[] = VTS_R8;
	sat.InvokeHelper(0xa, DISPATCH_METHOD, VT_R8, (void*)&result, parms, dateTime);
	return result;
}

JNIEXPORT jint JNICALL Java_jDJTSatLib_getRevNumber(JNIEnv *env, jobject obj)
{
	jint result;
	sat.InvokeHelper(11, DISPATCH_PROPERTYGET, VT_I4, (void*)&result, NULL);
	return result;
}

JNIEXPORT jdouble JNICALL Java_jDJTSatLib_getOrbitPhase(JNIEnv *env, jobject obj)
{
	jdouble result;
	sat.InvokeHelper(12, DISPATCH_PROPERTYGET, VT_R8, (void*)&result, NULL);
	return result;
}

JNIEXPORT jdouble JNICALL Java_jDJTSatLib_FindAOS(JNIEnv *env, jobject obj, jstring name, jdouble predictTime)
{
	jdouble result;
	LPCTSTR Name = env->GetStringUTFChars(name, NULL);
	static BYTE parms[] = VTS_BSTR VTS_R8;

	sat.InvokeHelper(13, DISPATCH_METHOD, VT_R8, (void*)&result, parms, Name, predictTime);
	env->ReleaseStringUTFChars(name, Name);

	return result;
}

JNIEXPORT jdouble JNICALL Java_jDJTSatLib_FindLOS(JNIEnv *env, jobject obj, jstring name, jdouble predictTime)
{
	jdouble result;
	LPCTSTR Name = env->GetStringUTFChars(name, NULL);
	static BYTE parms[] = VTS_BSTR VTS_R8;

	sat.InvokeHelper(14, DISPATCH_METHOD, VT_R8, (void*)&result, parms, Name, predictTime);
	env->ReleaseStringUTFChars(name, Name);

	return result;
}

JNIEXPORT jdouble JNICALL Java_jDJTSatLib_getMeanAnomaly(JNIEnv *env, jobject obj)
{
	jdouble result;
	sat.InvokeHelper(15, DISPATCH_PROPERTYGET, VT_R8, (void*)&result, NULL);
	return result;
}

JNIEXPORT void JNICALL Java_jDJTSatLib_SatState(JNIEnv *env, jobject obj, jdoubleArray x, jdoubleArray y, jdoubleArray z, jdoubleArray xDot, jdoubleArray yDot, jdoubleArray zDot)
{
	static BYTE parms[] = VTS_PR8 VTS_PR8 VTS_PR8 VTS_PR8 VTS_PR8 VTS_PR8;
	jdouble *X = env->GetDoubleArrayElements(x, 0);
	jdouble *Y = env->GetDoubleArrayElements(y, 0);
	jdouble *Z = env->GetDoubleArrayElements(z, 0);
	jdouble *Xdot = env->GetDoubleArrayElements(xDot, 0);
	jdouble *Ydot = env->GetDoubleArrayElements(yDot, 0);
	jdouble *Zdot = env->GetDoubleArrayElements(zDot, 0);

	sat.InvokeHelper(16, DISPATCH_METHOD, VT_EMPTY, NULL, parms, X, Y, Z, Xdot, Ydot, Zdot);

	env->ReleaseDoubleArrayElements(x, X, 0);
	env->ReleaseDoubleArrayElements(y, Y, 0);
	env->ReleaseDoubleArrayElements(z, Z, 0);
	env->ReleaseDoubleArrayElements(xDot, Xdot, 0);
	env->ReleaseDoubleArrayElements(yDot, Ydot, 0);
	env->ReleaseDoubleArrayElements(zDot, Zdot, 0);
}

JNIEXPORT jdouble JNICALL Java_jDJTSatLib_Period(JNIEnv *env, jobject obj, jstring name)
{
	double result;
	LPCTSTR Name = env->GetStringUTFChars(name, NULL);
	static BYTE parms[] = VTS_BSTR;

	sat.InvokeHelper(17, DISPATCH_METHOD, VT_R8, (void*)&result, parms, Name);
	env->ReleaseStringUTFChars(name, Name);

	return result;
}

JNIEXPORT void JNICALL Java_jDJTSatLib_KeplerData(JNIEnv *env, jobject obj, jstring name,
	jdoubleArray julianEpoch, jdoubleArray inclination, jdoubleArray raan, jdoubleArray eccentricity,
	jdoubleArray argOfPerigee, jdoubleArray decayRate, jdoubleArray period, jdoubleArray meanAnomaly)
{
	LPCTSTR Name = env->GetStringUTFChars(name, NULL);
	jdouble* JulianEpoch = env->GetDoubleArrayElements(julianEpoch, 0);
	jdouble* Inclination = env->GetDoubleArrayElements(inclination, 0);
	jdouble* RAAN = env->GetDoubleArrayElements(raan, 0);
	jdouble* Eccentricity = env->GetDoubleArrayElements(eccentricity, 0);
	jdouble* ArgOfPerigee = env->GetDoubleArrayElements(argOfPerigee, 0);
	jdouble* DecayRate = env->GetDoubleArrayElements(decayRate, 0);
	jdouble* Period = env->GetDoubleArrayElements(period, 0);
	jdouble* MeanAnomaly = env->GetDoubleArrayElements(meanAnomaly, 0);

	static BYTE parms[] = VTS_BSTR VTS_R8 VTS_R8 VTS_R8 VTS_R8 VTS_R8 VTS_R8 VTS_R8 VTS_R8;
	sat.InvokeHelper(18, DISPATCH_METHOD, VT_EMPTY, NULL, parms, Name, JulianEpoch, Inclination, RAAN, Eccentricity, ArgOfPerigee, DecayRate, Period, MeanAnomaly);

	env->ReleaseStringUTFChars(name, Name);
	env->ReleaseDoubleArrayElements(julianEpoch, JulianEpoch, 0);
	env->ReleaseDoubleArrayElements(inclination, Inclination, 0);
	env->ReleaseDoubleArrayElements(raan, RAAN, 0);
	env->ReleaseDoubleArrayElements(eccentricity, Eccentricity, 0);
	env->ReleaseDoubleArrayElements(argOfPerigee, ArgOfPerigee, 0);
	env->ReleaseDoubleArrayElements(decayRate, DecayRate, 0);
	env->ReleaseDoubleArrayElements(period, Period, 0);
	env->ReleaseDoubleArrayElements(meanAnomaly, MeanAnomaly, 0);
}


JNIEXPORT jstring JNICALL Java_jDJTSatLib_MakePassDetails(JNIEnv *env, jobject obj, jstring passDescription, jint stepSeconds)
{
	CString result;
	LPCTSTR PassDescription = env->GetStringUTFChars(passDescription, NULL);
	static BYTE parms[] = VTS_BSTR VTS_I4;

	sat.InvokeHelper(19, DISPATCH_METHOD, VT_BSTR, (void*)&result, parms, PassDescription, stepSeconds);
	env->ReleaseStringUTFChars(passDescription, PassDescription);

	return env->NewStringUTF(result.GetBuffer(0));;
}
